@@ -5,14 +5,13 @@ import numpy as np
 
 device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
 dtype = torch.float
-slope = 25
 
 
 class LIF(nn.Module):
     """Parent class for leaky integrate and fire neuron models."""
 
     instances = []
-    """Each :mod:`snntorch.LIF` neuron (e.g., :mod:`snntorch.Stein`) will populate the :mod:`snntorch.LIF.instances` list with a new entry.
+    """Each :mod:`snntorch.LIF` neuron (e.g., :mod:`snntorch.Synaptic`) will populate the :mod:`snntorch.LIF.instances` list with a new entry.
     The list is used to initialize and clear neuron states when the argument `init_hidden=True`."""
 
     def __init__(
@@ -70,25 +69,41 @@ class LIF(nn.Module):
         cls.instances = []
 
     @staticmethod
-    def init_stein(batch_size, *args):
+    def init_leaky(batch_size, *args):
+        """Used to initialize mem and spk.
+        *args are the input feature dimensions.
+        E.g., ``batch_size=128`` and input feature of size=1x28x28 would require ``init_leaky(128, 1, 28, 28)``."""
+        mem = torch.zeros((batch_size, *args), device=device, dtype=dtype)
+        spk = torch.zeros((batch_size, *args), device=device, dtype=dtype)
+
+        return spk, mem
+
+    @staticmethod
+    def init_synaptic(batch_size, *args):
         """Used to initialize syn, mem and spk.
         *args are the input feature dimensions.
-        E.g., ``batch_size=128`` and input feature of size=1x28x28 would require ``init_stein(128, 1, 28, 28)``."""
+        E.g., ``batch_size=128`` and input feature of size=1x28x28 would require ``init_synaptic(128, 1, 28, 28)``."""
         syn = torch.zeros((batch_size, *args), device=device, dtype=dtype)
         mem = torch.zeros((batch_size, *args), device=device, dtype=dtype)
         spk = torch.zeros((batch_size, *args), device=device, dtype=dtype)
 
         return spk, syn, mem
 
+    @staticmethod
+    def init_stein(batch_size, *args):
+        """Used to initialize syn, mem and spk.
+        *args are the input feature dimensions.
+        E.g., ``batch_size=128`` and input feature of size=1x28x28 would require ``init_stein(128, 1, 28, 28)``."""
+
+        return LIF.init_synaptic(batch_size, *args)
+
     @staticmethod
     def init_lapicque(batch_size, *args):
         """Used to initialize mem and spk.
         *args are the input feature dimensions.
         E.g., ``batch_size=128`` and input feature of size=1x28x28 would require ``init_lapicque(128, 1, 28, 28)``."""
-        mem = torch.zeros((batch_size, *args), device=device, dtype=dtype)
-        spk = torch.zeros((batch_size, *args), device=device, dtype=dtype)
 
-        return spk, mem
+        return LIF.init_leaky(batch_size, *args)
 
     @staticmethod
     def init_srm0(batch_size, *args):
@@ -153,10 +168,161 @@ class LIF(nn.Module):
 
 # Neuron Models
 
-
-class Stein(LIF):
+class Leaky(LIF):
     """
-    Stein's model of the leaky integrate and fire neuron.
+    First-order leaky integrate-and-fire neuron model.
+    Input is assumed to be a current injection.
+    Membrane potential decays exponentially with rate beta.
+    For :math:`U[T] > U_{\\rm thr} ⇒ S[T+1] = 1`.
+
+    If `reset_mechanism = "subtract"`, then :math:`U[t+1]` will have `threshold` subtracted from it whenever the neuron emits a spike:
+
+    .. math::
+
+            U[t+1] = βU[t] + I_{\\rm in}[t+1] - RU_{\\rm thr}
+
+    If `reset_mechanism = "zero"`, then :math:`U[t+1]` will be set to `0` whenever the neuron emits a spike:
+
+    .. math::
+
+            U[t+1] = βU[t] + I_{\\rm syn}[t+1] - R(βU[t] + I_{\\rm in}[t+1])
+
+    * :math:`I_{\\rm in}` - Input current
+    * :math:`U` - Membrane potential
+    * :math:`U_{\\rm thr}` - Membrane threshold
+    * :math:`R` - Reset mechanism: if active, :math:`R = 1`, otherwise :math:`R = 0`
+    * :math:`β` - Membrane potential decay rate
+
+    Example::
+
+        import torch
+        import torch.nn as nn
+        import snntorch as snn
+
+        alpha = 0.9
+        beta = 0.5
+
+        # Define Network
+        class Net(nn.Module):
+            def __init__(self):
+                super().__init__()
+
+                # initialize layers
+                self.fc1 = nn.Linear(num_inputs, num_hidden)
+                self.lif1 = snn.Leaky(beta=beta)
+                self.fc2 = nn.Linear(num_hidden, num_outputs)
+                self.lif2 = snn.Leaky(beta=beta)
+
+            def forward(self, x, mem1, spk1, mem2):
+                cur1 = self.fc1(x)
+                spk1, mem1 = self.lif1(cur1, mem1)
+                cur2 = self.fc2(spk1)
+                spk2, mem2 = self.lif2(cur2, mem2)
+                return mem1, spk1, mem2, spk2
+                
+        
+        """
+
+    def __init__(
+        self,
+        beta,
+        threshold=1.0,
+        num_inputs=False,
+        spike_grad=None,
+        batch_size=False,
+        hidden_init=False,
+        inhibition=False,
+        reset_mechanism="subtract",
+    ):
+        super(Leaky, self).__init__(
+            beta, threshold, spike_grad, inhibition, reset_mechanism
+        )
+
+        self.num_inputs = num_inputs
+        self.batch_size = batch_size
+        self.hidden_init = hidden_init
+
+        if self.hidden_init:
+            if not self.num_inputs:
+                raise ValueError(
+                    "num_inputs must be specified to initialize hidden states as instance variables."
+                )
+            elif not self.batch_size:
+                raise ValueError(
+                    "batch_size must be specified to initialize hidden states as instance variables."
+                )
+            elif hasattr(self.num_inputs, "__iter__"):
+                self.spk, self.mem = self.init_leaky(
+                    self.batch_size, *(self.num_inputs)
+                )  # need to automatically call batch_size
+            else:
+                self.spk, self.mem = self.init_leaky(
+                    self.batch_size, self.num_inputs
+                )
+        if self.inhibition:
+            if not self.batch_size:
+                raise ValueError(
+                    "batch_size must be specified to enable firing inhibition."
+                )
+
+    def forward(self, input_, mem):
+        if not self.hidden_init:
+            if self.inhibition:
+                spk, reset = self.fire_inhibition(self.batch_size, mem)
+            else:
+                spk, reset = self.fire(mem)
+
+            if self.reset_mechanism == "subtract":
+                mem = self.beta * mem + input_ - reset * self.threshold
+
+            elif self.reset_mechanism == "zero":
+                mem = self.beta * mem + input_ - reset * (self.beta * mem + input_)
+
+            return spk, mem
+
+        # intended for truncated-BPTT where instance variables are hidden states
+        if self.hidden_init:
+            if self.inhibition:
+                self.spk, self.reset = self.fire_inhibition(self.batch_size, self.mem)
+            else:
+                self.spk, self.reset = self.fire(self.mem)
+
+            if self.reset_mechanism == "subtract":
+                self.mem = self.beta * self.mem + input_ - self.reset * self.threshold
+
+            elif self.reset_mechanism == "zero":
+                self.mem = (
+                    self.beta * self.mem
+                    + input_
+                    - self.reset * (self.beta * self.mem + input_)
+                )
+
+            return self.spk, self.mem
+
+    @classmethod
+    def detach_hidden(cls):
+        """Returns the hidden states, detached from the current graph.
+        Intended for use in truncated backpropagation through time where hidden state variables are instance variables."""
+
+        for layer in range(len(cls.instances)):
+            if isinstance(cls.instances[layer], Leaky):
+                cls.instances[layer].spk.detach_()
+                cls.instances[layer].mem.detach_()
+
+    @classmethod
+    def zeros_hidden(cls):
+        """Used to clear hidden state variables to zero.
+        Intended for use where hidden state variables are instance variables."""
+
+        for layer in range(len(cls.instances)):
+            if isinstance(cls.instances[layer], Leaky):
+                cls.instances[layer].spk = torch.zeros_like(cls.instances[layer].spk)
+                cls.instances[layer].mem = torch.zeros_like(cls.instances[layer].mem)
+
+
+class Synaptic(LIF):
+    """
+    2nd order leaky integrate and fire neuron model accounting for synaptic conductance.
     The synaptic current jumps upon spike arrival, which causes a jump in membrane potential.
     Synaptic current and membrane potential decay exponentially with rates of alpha and beta, respectively.
     For :math:`U[T] > U_{\\rm thr} ⇒ S[T+1] = 1`.
@@ -199,9 +365,9 @@ class Stein(LIF):
 
                 # initialize layers
                 self.fc1 = nn.Linear(num_inputs, num_hidden)
-                self.lif1 = snn.Stein(alpha=alpha, beta=beta)
+                self.lif1 = snn.Synaptic(alpha=alpha, beta=beta)
                 self.fc2 = nn.Linear(num_hidden, num_outputs)
-                self.lif2 = snn.Stein(alpha=alpha, beta=beta)
+                self.lif2 = snn.Synaptic(alpha=alpha, beta=beta)
 
             def forward(self, x, syn1, mem1, spk1, syn2, mem2):
                 cur1 = self.fc1(x)
@@ -229,7 +395,7 @@ class Stein(LIF):
         inhibition=False,
         reset_mechanism="subtract",
     ):
-        super(Stein, self).__init__(
+        super(Synaptic, self).__init__(
             beta, threshold, spike_grad, inhibition, reset_mechanism
         )
 
@@ -248,11 +414,11 @@ class Stein(LIF):
                     "batch_size must be specified to initialize hidden states as instance variables."
                 )
             elif hasattr(self.num_inputs, "__iter__"):
-                self.spk, self.syn, self.mem = self.init_stein(
+                self.spk, self.syn, self.mem = self.init_synaptic(
                     self.batch_size, *(self.num_inputs)
                 )  # need to automatically call batch_size
             else:
-                self.spk, self.syn, self.mem = self.init_stein(
+                self.spk, self.syn, self.mem = self.init_synaptic(
                     self.batch_size, self.num_inputs
                 )
         if self.inhibition:
@@ -303,7 +469,7 @@ class Stein(LIF):
         Intended for use in truncated backpropagation through time where hidden state variables are instance variables."""
 
         for layer in range(len(cls.instances)):
-            if isinstance(cls.instances[layer], Stein):
+            if isinstance(cls.instances[layer], Synaptic):
                 cls.instances[layer].spk.detach_()
                 cls.instances[layer].syn.detach_()
                 cls.instances[layer].mem.detach_()
@@ -314,7 +480,7 @@ class Stein(LIF):
         Intended for use where hidden state variables are instance variables."""
 
         for layer in range(len(cls.instances)):
-            if isinstance(cls.instances[layer], Stein):
+            if isinstance(cls.instances[layer], Synaptic):
                 cls.instances[layer].spk = torch.zeros_like(cls.instances[layer].spk)
                 cls.instances[layer].syn = torch.zeros_like(cls.instances[layer].syn)
                 cls.instances[layer].mem = torch.zeros_like(cls.instances[layer].mem)
@@ -323,7 +489,7 @@ class Stein(LIF):
 class Lapicque(LIF):
     """
     An extension of Lapicque's experimental comparison between extracellular nerve fibers and an RC circuit.
-    It is equivalent to Stein's model without synaptic current dynamics.
+    It is qualitatively equivalent to :code:`Leaky` but defined using RC circuit parameters.
     Input stimulus is integrated by membrane potential which decays exponentially with a rate of beta.
     For :math:`U[T] > U_{\\rm thr} ⇒ S[T+1] = 1`.
 
@@ -799,3 +965,171 @@ class SRM0(LIF):
                     cls.instances[layer].syn_post
                 )
                 cls.instances[layer].mem = torch.zeros_like(cls.instances[layer].mem)
+
+
+class Stein(LIF):
+    """
+    Stein's model of the leaky integrate and fire neuron.
+    The synaptic current jumps upon spike arrival, which causes a jump in membrane potential.
+    Synaptic current and membrane potential decay exponentially with rates of alpha and beta, respectively.
+    For :math:`U[T] > U_{\\rm thr} ⇒ S[T+1] = 1`.
+
+    If `reset_mechanism = "subtract"`, then :math:`U[t+1]` will have `threshold` subtracted from it whenever the neuron emits a spike:
+
+    .. math::
+
+            I_{\\rm syn}[t+1] = αI_{\\rm syn}[t] + I_{\\rm in}[t+1] \\\\
+            U[t+1] = βU[t] + I_{\\rm syn}[t+1] - RU_{\\rm thr}
+
+    If `reset_mechanism = "zero"`, then :math:`U[t+1]` will be set to `0` whenever the neuron emits a spike:
+
+    .. math::
+
+            I_{\\rm syn}[t+1] = αI_{\\rm syn}[t] + I_{\\rm in}[t+1] \\\\
+            U[t+1] = βU[t] + I_{\\rm syn}[t+1] - R(βU[t] + I_{\\rm syn}[t+1])
+
+    * :math:`I_{\\rm syn}` - Synaptic current
+    * :math:`I_{\\rm in}` - Input current
+    * :math:`U` - Membrane potential
+    * :math:`U_{\\rm thr}` - Membrane threshold
+    * :math:`R` - Reset mechanism: if active, :math:`R = 1`, otherwise :math:`R = 0`
+    * :math:`α` - Synaptic current decay rate
+    * :math:`β` - Membrane potential decay rate
+
+    Example::
+
+        import torch
+        import torch.nn as nn
+        import snntorch as snn
+
+        alpha = 0.9
+        beta = 0.5
+
+        # Define Network
+        class Net(nn.Module):
+            def __init__(self):
+                super().__init__()
+
+                # initialize layers
+                self.fc1 = nn.Linear(num_inputs, num_hidden)
+                self.lif1 = snn.Stein(alpha=alpha, beta=beta)
+                self.fc2 = nn.Linear(num_hidden, num_outputs)
+                self.lif2 = snn.Stein(alpha=alpha, beta=beta)
+
+            def forward(self, x, syn1, mem1, spk1, syn2, mem2):
+                cur1 = self.fc1(x)
+                spk1, syn1, mem1 = self.lif1(cur1, syn1, mem1)
+                cur2 = self.fc2(spk1)
+                spk2, syn2, mem2 = self.lif2(cur2, syn2, mem2)
+                return syn1, mem1, spk1, syn2, mem2, spk2
+
+
+    For further reading, see:
+
+    *R. B. Stein (1965) A theoretical analysis of neuron variability. Biophys. J. 5, pp. 173-194.*
+
+    *R. B. Stein (1967) Some models of neuronal variability. Biophys. J. 7. pp. 37-68.*"""
+
+    def __init__(
+        self,
+        alpha,
+        beta,
+        threshold=1.0,
+        num_inputs=False,
+        spike_grad=None,
+        batch_size=False,
+        hidden_init=False,
+        inhibition=False,
+        reset_mechanism="subtract",
+    ):
+        super(Stein, self).__init__(
+            beta, threshold, spike_grad, inhibition, reset_mechanism
+        )
+
+        print("`Stein` has been deprecated and will be removed in a future version. Use `Synaptic` instead.")
+
+        self.alpha = alpha
+        self.num_inputs = num_inputs
+        self.batch_size = batch_size
+        self.hidden_init = hidden_init
+
+        if self.hidden_init:
+            if not self.num_inputs:
+                raise ValueError(
+                    "num_inputs must be specified to initialize hidden states as instance variables."
+                )
+            elif not self.batch_size:
+                raise ValueError(
+                    "batch_size must be specified to initialize hidden states as instance variables."
+                )
+            elif hasattr(self.num_inputs, "__iter__"):
+                self.spk, self.syn, self.mem = self.init_stein(
+                    self.batch_size, *(self.num_inputs)
+                )  # need to automatically call batch_size
+            else:
+                self.spk, self.syn, self.mem = self.init_stein(
+                    self.batch_size, self.num_inputs
+                )
+        if self.inhibition:
+            if not self.batch_size:
+                raise ValueError(
+                    "batch_size must be specified to enable firing inhibition."
+                )
+
+    def forward(self, input_, syn, mem):
+        if not self.hidden_init:
+            if self.inhibition:
+                spk, reset = self.fire_inhibition(self.batch_size, mem)
+            else:
+                spk, reset = self.fire(mem)
+            syn = self.alpha * syn + input_
+
+            if self.reset_mechanism == "subtract":
+                mem = self.beta * mem + syn - reset * self.threshold
+
+            elif self.reset_mechanism == "zero":
+                mem = self.beta * mem + syn - reset * (self.beta * mem + syn)
+
+            return spk, syn, mem
+
+        # intended for truncated-BPTT where instance variables are hidden states
+        if self.hidden_init:
+            if self.inhibition:
+                self.spk, self.reset = self.fire_inhibition(self.batch_size, self.mem)
+            else:
+                self.spk, self.reset = self.fire(self.mem)
+            self.syn = self.alpha * self.syn + input_
+
+            if self.reset_mechanism == "subtract":
+                self.mem = self.beta * self.mem + self.syn - self.reset * self.threshold
+
+            elif self.reset_mechanism == "zero":
+                self.mem = (
+                    self.beta * self.mem
+                    + self.syn
+                    - self.reset * (self.beta * self.mem + self.syn)
+                )
+
+            return self.spk, self.syn, self.mem
+
+    @classmethod
+    def detach_hidden(cls):
+        """Returns the hidden states, detached from the current graph.
+        Intended for use in truncated backpropagation through time where hidden state variables are instance variables."""
+
+        for layer in range(len(cls.instances)):
+            if isinstance(cls.instances[layer], Stein):
+                cls.instances[layer].spk.detach_()
+                cls.instances[layer].syn.detach_()
+                cls.instances[layer].mem.detach_()
+
+    @classmethod
+    def zeros_hidden(cls):
+        """Used to clear hidden state variables to zero.
+        Intended for use where hidden state variables are instance variables."""
+
+        for layer in range(len(cls.instances)):
+            if isinstance(cls.instances[layer], Stein):
+                cls.instances[layer].spk = torch.zeros_like(cls.instances[layer].spk)
+                cls.instances[layer].syn = torch.zeros_like(cls.instances[layer].syn)
+                cls.instances[layer].mem = torch.zeros_like(cls.instances[layer].mem)
